import os
import re
import argparse
import logging

import opencc

from Librian虛擬機 import 讀者
from Librian虛擬機 import 角色

cc = opencc.OpenCC('t2s')

此處 = os.path.dirname(os.path.abspath(__file__))


class 虛狀態:
    def __init__(self):
        self.額外信息 = ()
        self.內容 = {}

    def 導出(self):
        return {'內容': self.內容, '額外信息': self.額外信息}


class 虛讀者(讀者.讀者):
    def __init__(self, 初始劇本, 簡化字=False):
        super().__init__(初始劇本)
        self.簡化字 = 簡化字

    def 步進(self):
        self.狀態 = 虛狀態()
        s = self.下一句()
        類型 = s['類型']
        if 類型 == '終焉':
            self.狀態.額外信息 = ('終焉',)
        if 類型 in ('註釋', '躍點'):
            self.狀態.內容 = f'<span class="{類型}">{s[類型]}</span>\n'
        if 類型 == '函數調用':
            self.狀態.內容 = f'<span class="函數調用">{s["原文"]}</span>\n'
        if 類型 == '插入圖':
            self.狀態.內容 = f'<div class="插入">{s["插入圖"]}</div>'
        if 類型 == '旁白':
            旁白 = self.打標點(self.化(s["旁白"]))
            self.狀態.內容 = f'<p class="旁白">{旁白}</p>\n'
        if 類型 == '人物操作':
            人物名 = s['人物名']
            目標 = s['目標']
            if s['操作符'] == '+':
                self.狀態.內容 = f'<p class="人物操作">{self.化(人物名)} + {self.化(目標)}</p>\n'
            if s['操作符'] == '|':
                角色.取角色(人物名).顯示名字 = s['目標']
                self.狀態.內容 = ''
        if 類型 in ('人物對話', '人物表情'):
            替代顯示名字 = 角色.取角色(s['名']).顯示名字
            名字 = s['代'] or 替代顯示名字 or s['名']
            self.狀態.內容 = f'<p><span class="代 {s["名"]}">{self.化(名字)}</span>'
            if s['顏']:
                self.狀態.內容 += f'<span class="顏">{s["顏"]}</span>'
            if 類型 == '人物對話':
                self.狀態.內容 += f'<span class="語">{self.化(s["語"])}</span>'
            self.狀態.內容 += f'</p>'

        if '之後的空白' in s:
            self.狀態.內容 += '<br/>\n' * s['之後的空白']

    def 化(self, s):
        if self.簡化字:
            return cc.convert(s)
        return s

    def 打標點(self, 句):
        句 = re.sub('([，。？！…——])', '<span class="標點">\\1</span>', 句)
        句 = re.sub('(「.*?」)', '<span class="引用">\\1</span>', 句)
        return 句

    def 導出html(self, 帶css=False):
        head = f'<meta charset="utf8"/>\n'
        if 帶css:
            for css in 參數.css:
                head += f'<link rel="stylesheet" href="{css}"/>\n'

        return head + self.body內容()

    def body內容(self):
        t = list(self.迭代器())
        return '\n'.join([i['內容'] for i in t])


if __name__ == '__main__':
    參數 = argparse.ArgumentParser(description='劇本文件生成pdf')
    參數.add_argument('--play', type=str, required=True)
    參數.add_argument('--css', type=str, action='append')
    參數.add_argument('--out', type=str)
    參數.add_argument('--chs', action='store_true')
    參數.add_argument('--html', action='store_true')
    參數 = 參數.parse_args()

    if not 參數.css:
        參數.css = [os.path.join(此處, './資源/導出pdf用/紙樣式.css')]

    讀 = 虛讀者(參數.play, 參數.chs)
    if 參數.html:
        with open(參數.out, 'w', encoding='utf8') as f2:
            f2.write(讀.導出html(帶css=True))
    else:
        from weasyprint import HTML, CSS
        HTML(string=讀.導出html()).write_pdf(參數.out, stylesheets=參數.css)



import logging
import os

import yaml
yaml.warnings({'YAMLLoadWarning': False})


def 導入全局配置(a):
    配置.update(a)
    logging.debug(配置)


with open(os.path.split(os.path.realpath(__file__))[0] + '/配置.yaml', encoding='utf8') as f:
    配置 = yaml.load(f)
    logging_config = {
        'format': '【%(filename)s - %(lineno)s】(%(levelname)s): %(message)s',
    }
    if 配置['額外信息']:
        logging_config['level'] = logging.INFO
    logging.basicConfig(**logging_config)



import logging

from librian.librian_util import 文件
try:
    from rimo_utils.cef_tools import wxcef
except ModuleNotFoundError:
    logging.warning('沒能import wx，改爲使用pyside2。')
    from rimo_utils.cef_tools import qtcef as wxcef

from librian.librian_util import 路徑

from . import 山彥
from .環境 import 配置

from .librian虛擬機 import 虛擬機環境



def 啓動app():
    if 配置['編寫模式']:
        url = 路徑.librian本體 / '前端/adv.html'
    elif 虛擬機環境.標題畫面:
        url = f'{虛擬機環境.工程路徑}/{虛擬機環境.標題畫面}'
    else:
        url = 路徑.librian本體 / '前端/默認標題畫面/標題.html'

    標題url = 文件.轉爲網址路徑(url)

    if 虛擬機環境.圖標:
        圖標 = f'{虛擬機環境.工程路徑}/{虛擬機環境.圖標}'
    else:
        圖標 = 路徑.librian本體 / '資源/librian.ico'

    app, 瀏覽器 = wxcef.group(
        title=虛擬機環境.標題, 
        url=標題url, 
        icon=圖標, 
        size=虛擬機環境.主解析度
    )
    山彥.綁定(app, 標題url=標題url)

    return app



import logging
import os

import liber

from . import 讀者


class 箱庭:
    def __init__(self, 關聯讀者):
        self.關聯讀者 = 關聯讀者
        self.箱庭內容 = {
            'goto': self.跳轉,
            'call': self.棧跳轉,
            'choice': self.產生選項,
            'fusion': self.同化,
            'adv_end': self.演出終了,
            
            '跳轉': self.跳轉,
            '調用': self.棧跳轉,
            '產生選項': self.產生選項,
            '同化': self.同化,
            '演出終了': self.演出終了,
            
            '跳转': self.跳轉,
            '调用': self.棧跳轉,
            '产生选项': self.產生選項,
            '同化': self.同化,
            '演出终了': self.演出終了,
        }

    def 跳轉(self, path=None, lable=None, 彈=True):
        現名 = self.關聯讀者.劇本文件.名
        if not path:
            path = 現名
        else:
            path = '%s/%s' % (os.path.dirname(現名), path)

        if 彈:
            self.關聯讀者.劇本棧.pop()
        self.關聯讀者.劇本棧.append(self.關聯讀者.編譯(path))
        if lable:
            while True:
                t = self.關聯讀者.劇本文件.下一句()
                if t is None:
                    raise Exception(f'沒有找到躍點「{lable}」')
                if '躍點' in t and t['躍點'] == lable:
                    break

    def 棧跳轉(self, path=None, lable=None):
        self.跳轉(path, lable, 彈=False)

    def 產生選項(self, *li):
        li = [(i[0], i[1]) for i in li]
        self.關聯讀者.狀態.選項 = li

    def 同化(self, s):
        self.關聯讀者.劇本棧.append(讀者.劇本(liber.load(s), '_字串'))

    def 演出終了(self):
        self.關聯讀者.劇本棧 = []



import logging

from . import 立繪
from . import 虛擬機環境

語者 = ''

鏡頭對應 = {}

上次位置 = {}


class 鏡頭:
    def __init__(self, 所有位置):
        self.所有位置 = 所有位置
        for 人, 位置 in 所有位置.items():
            鏡頭對應[人] = self
            if len(位置) == 2:
                位置.append(1)

    def 統合(self):
        global 上次位置
        t = {}
        for 人 in self.所有位置:
            位置 = self.所有位置[人]
            t[人] = {
                '位置': 位置,
                '語': 人 == 語者
            }
        上次位置 = {}
        for 人 in self.所有位置:
            上次位置[人] = self.所有位置[人]
        return t

    def 拆解(self):
        立繪組 = [立繪.人物拆解(人, 參數) for 人, 參數 in self.統合().items()]
        return [立繪 for 立繪 in 立繪組 if 立繪 is not None]

    def __repr__(self):
        return str(self.所有位置)

    def __bool__(self):
        return bool(self.所有位置)


空鏡頭 = 鏡頭({})


def 查詢(人):
    if 人 not in 鏡頭對應:
        鏡頭({人: 虛擬機環境.默認立繪位置[1][0]})
    return 鏡頭對應[人]

# ['潘大爺'] -> {'潘大爺':[0,0]}


def 生成鏡頭(x):
    if type(x) == dict:
        return 鏡頭(x)
    if type(x) == list:
        m = 虛擬機環境.默認立繪位置[len(x)]
        a = dict(zip(x, m))
        return 鏡頭(a)


def 解除鏡頭(人):
    鏡頭對應[人] = 空鏡頭


if __name__ == '__main__':
    print(查詢('潘大爺'))
    print(空鏡頭)



import os
import logging

from . import 環境
from . import librian虛擬機
from . import 窗口

from .librian虛擬機 import 虛擬機環境


def ember(project, 編寫模式=False):
    logging.info('librian_main啓動。')
    librian虛擬機.虛擬機環境.加載配置(project)
    環境.導入全局配置({
        '編寫模式': 編寫模式
    })

    os.makedirs(f'{虛擬機環境.工程路徑}/存檔資料/手動存檔', exist_ok=True)
        
    app = 窗口.啓動app()
    app.MainLoop()



import os
import json
import logging
import time
import datetime
import pickle

import yaml
from rimo_utils import good_open
from rimo_utils.cef_tools.vue_ob import vue_ob

from librian.librian_util import 文件, 加載器, 路徑

from .librian虛擬機 import 讀者
from .librian虛擬機 import 虛擬機環境

from .環境 import 配置


def 綁定(app, 標題url):
    讀者實例 = 讀者.讀者(f'{虛擬機環境.工程路徑}/{虛擬機環境.劇本入口}')
    app.frame.set_browser_object("山彥", 極山彥(app.frame, app.frame.browser, 讀者實例, 標題url))


class 山彥(vue_ob):
    def __init__(self, 窗口, 瀏覽器, 讀者, 標題url):
        super().__init__()
        self.窗口 = 窗口
        self.瀏覽器 = 瀏覽器
        self.讀者 = 讀者
        self.標題url = 標題url

    def js(self, x):
        self.瀏覽器.ExecuteJavascript(x)

    def 取檔(self, callback):
        檔表 = os.listdir(f'{虛擬機環境.工程路徑}/存檔資料/手動存檔')
        檔表 = sorted(檔表, reverse=True)
        檔信息表 = []
        for i in 檔表:
            try:
                with open(f'{虛擬機環境.工程路徑}/存檔資料/手動存檔/{i}', 'rb') as f:
                    存檔信息 = pickle.load(f)['存檔信息']
                    檔信息表.append({
                        '名字': i,
                        '描述': 存檔信息['描述'],
                        '截圖': 存檔信息['截圖'],
                        '時間': datetime.datetime.fromtimestamp(存檔信息['存檔時間']).strftime('%Y-%m-%d %H:%M'),
                    })
            except Exception as e:
                logging.exception(e)
                logging.warning(f'存檔「{i}」有問題。')
        callback.Call(檔信息表)

    def 存檔(self, 文件名, 描述, 截圖):
        存檔信息 = {
            '描述': 描述,
            '截圖': 截圖,
            '存檔時間': time.time(),
        }
        self.讀者.存檔(f'{虛擬機環境.工程路徑}/存檔資料/手動存檔/{文件名}', 存檔信息)

    def 讀檔(self, 文件名):
        self.讀者.讀檔(f'{虛擬機環境.工程路徑}/存檔資料/手動存檔/{文件名}')
        self.更新(瞬間化=True)

    def 快速存檔(self):
        self.讀者.存檔(f'{虛擬機環境.工程路徑}/存檔資料/快速存檔.pkl')
        self.js('_py演出.提示("存檔好了。")')

    def 快速讀檔(self):
        self.讀者.讀檔(f'{虛擬機環境.工程路徑}/存檔資料/快速存檔.pkl')
        self.更新(瞬間化=True)

    def 切換全屏(self):
        self.窗口.toggleFullScreen()
        
    def 退出(self):
        exit()

    def vue更新(self, 內容):
        t = self.vue.用戶設置 if '用戶設置' in self.vue._內容 else None
        if t != 內容['用戶設置']:
            with open(f'{虛擬機環境.工程路徑}/存檔資料/用戶設置.yaml', 'w', encoding='utf8') as f:
                f.write(yaml.dump(內容['用戶設置']))
        super().vue更新(內容)


class 演出山彥(山彥):
    def 回標題(self):
        self.讀者.__init__(f'{虛擬機環境.工程路徑}/{虛擬機環境.劇本入口}')
        self.js(f'window.location.href={self.標題url.__repr__()}')

    def 步進(self):
        if 配置['編寫模式']:
            return
        self.讀者.步進()

    def 更新(self, 瞬間化=False):
        狀態 = self.讀者.狀態.導出()
        self.js(f'_py演出.改變演出狀態({json.dumps(狀態)},{json.dumps(瞬間化)})')

    def 狀態回調(self, 步進, callback):
        if 步進:
            self.步進()
        狀態 = self.讀者.狀態.導出()
        callback.Call(狀態)

    def 初始化(self, callback):
        self.vue.圖片文件夾 = 虛擬機環境.工程路徑 / 虛擬機環境.圖片文件夾
        self.vue.音樂文件夾 = 虛擬機環境.工程路徑 / 虛擬機環境.音樂文件夾
        self.vue.視頻文件夾 = 虛擬機環境.工程路徑 / 虛擬機環境.視頻文件夾
        self.vue.psd立繪路徑 = 虛擬機環境.psd立繪路徑
        self.vue.自定css = [虛擬機環境.工程路徑 / i for i in 虛擬機環境.自定css]
        self.vue.主題css = os.path.join('主題', 虛擬機環境.主題css + '.css').replace('\\', '/')
        self.vue.解析度 = 虛擬機環境.主解析度
        self.vue.邊界 = 配置['顯示繪圖邊界']
        self.vue.翻譯 = 虛擬機環境.翻譯

        用戶設置 = 加載器.yaml(f'{虛擬機環境.工程路徑}/存檔資料/用戶設置.yaml')
        if 用戶設置:
            self.vue.用戶設置 = 用戶設置
        callback.Call()

    def 選(self, 參數):
        t = self.讀者.狀態.選項[參數][1]
        self.讀者.狀態.選項 = []
        t()
        logging.debug(f'選擇了「{參數}」。')
        self.讀者.步進()
        self.更新()


class 帶標題山彥(演出山彥):
    def 開始(self):
        self.步進()
        self.js(f'window.location.href={文件.轉爲網址路徑(路徑.librian本體 / "前端/adv.html").__repr__()};')

    def 讀檔畫面(self):
        self.js(f'window.location.href={(文件.轉爲網址路徑(路徑.librian本體 / "./前端/adv.html")+"?入口=讀檔").__repr__()};')

    def 從劇本開始(self, 劇本):
        入口 = f'{虛擬機環境.工程路徑}/{虛擬機環境.劇本入口}'
        self.讀者.__init__(f'{os.path.dirname(入口)}/{劇本}')
        self.開始()


class 極山彥(帶標題山彥):
    def __init__(self, *li, **d):
        super().__init__(*li, **d)
        if 配置['編寫模式']:
            import threading
            
            def 監視():
                原字 = ''
                while True:
                    with good_open(f'{虛擬機環境.工程路徑}/{虛擬機環境.劇本入口}') as f:
                        字 = f.read()
                        if 字 != 原字:
                            self.更新終態()
                            原字 = 字
            t = threading.Thread(target=監視)
            t.setDaemon(True)
            t.start()

    def 更新終態(self):
        self.讀者.從一而終(f'{虛擬機環境.工程路徑}/{虛擬機環境.劇本入口}')
        self.更新()

    def 初始化(self, *li, **d):
        super().初始化(*li, **d)
        if 配置['編寫模式']:
            self.更新終態()



import os
import json
from pathlib import Path
import shutil
from .環境 import 配置

from librian.librian_util import 路徑

from .librian虛擬機 import 虛擬機環境
from .librian虛擬機 import 讀者



def 演出固化(劇本文件):
    讀者實例 = 讀者.讀者(劇本文件)
    演出步 = list(讀者實例.迭代器())
    return 演出步


def 生成虛擬核心():
    圖片文件夾 = 路徑.虛擬相對前端路徑(虛擬機環境.圖片文件夾)
    音樂文件夾 = 路徑.虛擬相對前端路徑(虛擬機環境.音樂文件夾)
    視頻文件夾 = 路徑.虛擬相對前端路徑(虛擬機環境.視頻文件夾)
    psd立繪路徑 = 路徑.虛擬相對前端路徑(os.path.relpath(虛擬機環境.psd立繪路徑, start=虛擬機環境.工程路徑))
    自定css = [路徑.虛擬相對前端路徑(i) for i in 虛擬機環境.自定css]
    主題css = os.path.join('主題', 虛擬機環境.主題css + '.css').replace('\\', '/')

    演出步 = 演出固化(f'{虛擬機環境.工程路徑}/{虛擬機環境.劇本入口}')

    虛擬核心 = {
        '作品名': 虛擬機環境.標題,
        '解析度': 虛擬機環境.主解析度,
        '邊界': int(配置["顯示繪圖邊界"]),
        '圖片文件夾': 圖片文件夾,
        '音樂文件夾': 音樂文件夾,
        '視頻文件夾': 視頻文件夾,
        'psd立繪路徑': psd立繪路徑,
        '自定css': 自定css,
        '主題css': 主題css,
        '演出步': 演出步
    }
    json數據 = json.dumps(虛擬核心, indent=2, ensure_ascii=False)
    return f'window.虛擬核心 = {json數據}'


def 幻象化(目標路徑):
    目標路徑 = Path(目標路徑)
    依賴 = [
        '黑科技',
        'librian/librian本體/前端/dist',
        'librian/librian本體/前端/素材',
        'librian/librian本體/前端/主題',
        'librian/librian本體/前端/adv.html',
        虛擬機環境.工程路徑.relative_to(路徑.librian外層),
    ]
    for i in 依賴:
        源路徑 = 路徑.librian外層 / i
        if 源路徑.is_dir():
            shutil.copytree(源路徑, 目標路徑 / i)
        elif 源路徑.is_file():
            shutil.copy(源路徑, 目標路徑 / i)
        else:
            raise Exception('哈？')

    with open(目標路徑 / 'librian/librian本體/前端/虛擬核心.js', 'w', encoding='utf-8') as f:
        f.write(生成虛擬核心())

    with open(目標路徑 / '說明.txt', 'w', encoding='utf-8') as f:
        f.write('因爲它有一些奇怪的操作所以不能直接拉進瀏覽器運行。\n你需要自建服務器，一個可行的方法是安裝http-server，在這個路徑啓動。然後「http://localhost:8080/librian/librian本體/前端/adv.html」。')



import os
from pathlib import Path


此處 = Path(__file__).parent

librian = (此處 / '..').resolve()

librian面板 = librian / 'librian面板'
librian本體 = librian / 'librian本體'



def 虛擬相對前端路徑(x):
    源 = librian本體 / 'v' / x
    前端 = librian本體 / '前端'
    return os.path.relpath(源, start=前端)



import 環境
import os
環境.設定工程路徑('./project/極夜所在的星之海')

import 立繪
from psd生成html import 生成html

格寬=315
格高=350
縮放=0.9

tot=''
tot+=f'''
<style>
#bg{{
    background: url('static/紙背景.jpg');
    background-size: cover;
    position:fixed;
    top:0;
    left:0;
    width: 100%;
    height: 100%;
    margin: 0;
    padding: 0;
    z-index: -1;
}}
.外格{{
    width:{格寬*縮放};
    height:{格高*縮放};
    posision:relative;
    display:inline-block;
    border:1px solid #888;
    overflow:hidden;
}}
.格{{
    transform-origin:top left;
    transform:scale({縮放}) translate(-250px,-100px);
}}
.字{{
    height:0;
    posision:absolute;
    display:inline-block;
}}
.True{{
    opacity: 0; 
    background:rgba(0,0,0,0);
    transition: opacity 0.5s;
    filter:drop-shadow(5px 5px 5px #ccc);
}}
.True:hover{{
    opacity: 1; 
    background:rgba(0,0,0,0.5);
}}
</style>
<div id="bg"></div>
'''
for 名,人 in 立繪.映射.items():
    for 顏 in 人['顏']:
        if type(人['顏'][顏])==dict:
            能語=[False,True]
        else:
            能語=[False]
            
        t=''
        for 語 in 能語: 
            包 = 立繪.人物拆解(名,{'顏':顏,'衣':'_默認','語':語,'位置':[0,0,1],'動作':[]})
            t+=f'<div class="格 {語}">{生成html(包)}</div>'
            
        tot+=f'''
        <div class="外格">
            <div class="字"> 
                {名} {顏}
            </div> 
            {t}
        </div>
        '''
    tot+='<br/>'

with open('./html/_立繪監視.html','w',encoding='utf8') as f:
    f.write(tot)
    
os.system('start "" chrome ./html/_立繪監視.html')


import logging
import sys
from pathlib import Path

import yaml


class 假面:
    配置 = {}

    def 加載配置(self, 工程路徑):
        工程路徑 = Path(工程路徑)
        with open(工程路徑 / '工程配置.yaml', encoding='utf8') as f:
            a = yaml.safe_load(f)
            for i in a:
                self.配置.update(a[i])
        self.配置['工程路徑'] = 工程路徑
        self.配置['psd立繪路徑'] = 工程路徑 / self.配置['立繪文件夾']

    def __getattr__(self, x):
        return self.配置[x]


sys.modules[__name__] = 假面()



import logging

from librian.librian_util import 文件

from . import 虛擬機環境


def 別名適用(x):
    for i, f in list(x.__dict__.items()):
        if '別名' in dir(f):
            for 各別名 in f.別名:
                x.__class__.__setattr__(x, 各別名, f)
    return x


def 別名(*li):
    def f2(x):
        x.別名 = li
        return x
    return f2


@別名適用
class 命令:
    def __init__(self, 函數, 參數表, 原文):
        self.函數 = 函數
        self.參數 = [i['a'] for i in 參數表]
        self.原文 = 原文

    def 執行(self, 讀者):
        try:
            t = eval(self.原文, {
                i: (lambda *li, f=f: f(self, 讀者, *li))
                for i, f in self.__class__.__dict__.items()
                if i not in ('py', 'js') and i[0] != '_'
            })
            if t is not None:
                raise Exception('不太對勁……')
        except:
            try:
                函數 = eval('self.' + self.函數)
                函數(*([讀者] + self.參數))
            except AttributeError as e:
                raise AttributeError('沒有可用的函數「%s」。' % self.函數)
            except Exception as e:
                s = '%s(%s)' % (self.函數, ', '.join(self.參數))
                logging.warning(f'在劇本中執行方法「{s}」時遇到了意外。')
                logging.exception(e)

    # ——————————————————————————————

    @別名('背景')
    def BG(self, 讀者, bg, 淡入時間=1, 位置='0% 0%', 漸變方法='_淡出'):
        bg = 文件.補充擴展名(bg, ['webp', 'png', 'jpg'], 虛擬機環境.工程路徑 / 虛擬機環境.圖片文件夾)
        讀者.狀態.背景 = bg, 淡入時間, 位置, 漸變方法

    @別名('特效')
    def EF(self, 讀者, 標識, 類名=None):
        if 類名 is None and 標識 in 讀者.狀態.特效表:
            del 讀者.狀態.特效表[標識]
        else:
            讀者.狀態.特效表[標識] = 類名

    @別名('背景音樂', '背景音乐')
    def BGM(self, 讀者, 文件名, 音量=1):
        if 文件名 is None or 文件名 == 'None':
            讀者.狀態.背景音樂 = None
        else:
            文件名 = 文件.補充擴展名(文件名, ['opus', 'mp3', 'ogg'], 虛擬機環境.工程路徑 / 虛擬機環境.音樂文件夾)
            讀者.狀態.背景音樂 = 文件名, 音量

    def CG(self, 讀者, cg, 淡入時間=1, 漸變方法='_淡出'):
        if cg is None or cg == 'None':
            讀者.狀態.cg = None
        else:
            cg = 文件.補充擴展名(cg, ['webp', 'png', 'jpg'], 虛擬機環境.工程路徑 / 虛擬機環境.圖片文件夾)
            讀者.狀態.cg = cg, 淡入時間, 漸變方法

    @別名('效果音')
    def SE(self, 讀者, 文件名, 音量=1):
        文件名 = 文件.補充擴展名(文件名, ['opus', 'mp3', 'ogg'], 虛擬機環境.工程路徑 / 虛擬機環境.音樂文件夾)
        讀者.狀態.效果音 = 文件名, 音量

    @別名('視頻', '视频')
    def VIDEO(self, 讀者, 文件名, 可以跳過=False):
        讀者.狀態.重置()
        讀者.狀態.視頻 = 文件名, 可以跳過



import unittest

from rimo_utils import good_open

import 環境
from Librian虛擬機 import 虛擬機環境

虛擬機環境.加載配置('./project/_默認工程')


class 全部測試(unittest.TestCase):
    maxDiff = None
    # def test_角色(self):
    #     from Librian虛擬機 import 角色
    #     self.assertEqual(
    #         str(角色.角色表),
    #         "{'潘大爺': 角色(潘大爺->[衣:{'_默認': ['_']}],[顏:{'_默認': []}])}"
    #     )
    #     self.assertEqual(
    #         str(角色.角色表['潘大爺'].圖層座標),
    #         "{'_': {'x': 0, 'y': 0}}"
    #     )

    def test_編譯(self):
        from Librian虛擬機 import 編譯
        with good_open(f'{虛擬機環境.工程路徑}/{虛擬機環境.配置["劇本入口"]}') as f:
            self.assertEqual(
                編譯.編譯(f),
                [
                    {'縮進數': 0, '註釋': ' 背景來自pixiv_id=75399001的「無料背景素材[和風縁側]」。', '類型': '註釋'},
                    {'縮進數': 0, '註釋': ' 潘大爺的立繪來自某黃昏風立繪生成器。', '類型': '註釋'}, {'縮進數': 0, '註釋': ' 音樂來自https://filmmusic.io，作者Kevin MacLeod，使用CC4條款。', '類型': '註釋', '之後的空白': 1},
                    {'縮進數': 0, '原文': 'BGM call-to-adventure', '函數': 'BGM', '參數表': [{'a': 'call-to-adventure'}], '類型': '函數調用'},
                    {'縮進數': 0, '原文': 'BG 和風縁側', '函數': 'BG', '參數表': [{'a': '和風縁側'}], '類型': '函數調用'},
                    {'縮進數': 0, '名': '潘大爺', '代': None, '特效': None, '顏': None, '語': '今天天氣不錯，\n去散步吧。', '類型': '人物對話'},
                    {'縮進數': 0, '類型': '旁白', '旁白': '潘大爺走了。'}, {'縮進數': 0, '鏡頭符號': '-', '內容': '潘大爺', '類型': '鏡頭'}, {'縮進數': 0, '類型': '旁白', '旁白': '然後就誰也沒有了。'}
                ]
            )

    def test_劇本(self):
        from Librian虛擬機 import 劇本
        讀者 = 劇本.讀者(f'{虛擬機環境.工程路徑}/{虛擬機環境.配置["劇本入口"]}')
        self.assertEqual(
            list(讀者.迭代器()),
            [
                {'額外信息': '', '話語': '今天天氣不錯，\n去散步吧。', '名字': '潘大爺', '立繪': [{'圖層': [{'文件': '../project/_默認工程/_臨時文件/潘大爺/_.png', '子位置': (0, 0)}], '位置': [300, 0, 1], '動作': ('淡入',), '特效': None, '名字': '潘大爺'}], '視頻': '', '背景': ('和風縁側.jpg', 1, '_淡出'), '背景音樂': ('call-to-adventure.mp3', 1), 'cg': '', 'js': '', '選項': [], '語者': '潘大爺', '特效表': {}},
                {'額外信息': '', '話語': '潘大爺走了。', '名字': '', '立繪': [{'圖層': [{'文件': '../project/_默認工程/_臨時文件/潘大爺/_.png', '子位置': (0, 0)}], '位置': [300, 0, 1], '動作': None, '特效': None, '名字': '潘大爺'}], '視頻': '', '背景': ('和風縁側.jpg', 1, '_淡出'), '背景音樂': ('call-to-adventure.mp3', 1), 'cg': '', 'js': '', '選項': [], '語者': '', '特效表': {}},
                {'額外信息': '', '話語': '然後就誰也沒有了。', '名字': '', '立繪': [], '視頻': '', '背景': ('和風縁側.jpg', 1, '_淡出'), '背景音樂': ('call-to-adventure.mp3', 1), 'cg': '', 'js': '', '選項': [], '語者': '', '特效表': {}}
            ]
        )

unittest.main()


import os
import logging


def 補充擴展名(文件名, 擴展名表, 路徑='./'):
    if '.' in os.path.split(文件名)[-1]:
        if not os.path.isfile(f'{路徑}/{文件名}'):
            logging.warning(f'「{文件名}」不存在。')
        return 文件名
    for 擴展名 in 擴展名表:
        if os.path.isfile(f'{路徑}/{文件名}.{擴展名}'):
            return f'{文件名}.{擴展名}'
    logging.warning(f'「{文件名}」不存在。')
    return 文件名

def 轉爲網址路徑(路徑):
    絕對路徑 = os.path.abspath(路徑)
    return f'file:///{絕對路徑}'


import os
import logging

import yaml
from rimo_utils import good_open

from . import 虛擬機環境


角色表 = {}


def 初始化():    
    if '了' in dir(初始化):
        return
    初始化.了 = True
    導入有立繪的角色()


def 取角色(名字):
    初始化()
    if 名字 in 角色表:
        return 角色表[名字]
    else:
        return 角色(名字)


def 導入有立繪的角色():
    try:
        with good_open(虛擬機環境.psd立繪路徑 / '映射.yaml') as f:
            映射表 = yaml.load(f, Loader=yaml.BaseLoader)
            if 映射表:
                for 角色名 in 映射表:
                    角色(角色名, 映射表[角色名])
            for i in os.listdir(虛擬機環境.psd立繪路徑):
                if i.endswith('.png'):
                    前名 = os.path.basename(os.path.splitext(i)[0])
                    角色(前名, 使用png=True)
    except Exception as e:
        logging.warning('角色立繪沒有導入。')
        logging.exception(e)


class 角色:
    def __init__(self, 名字: str, 立繪表: dict = None, 使用png=False):
        self.名字 = 名字
        self.顯示名字 = None
        self.使用png = 使用png
        self.有立繪 = bool(立繪表 or 使用png)
        if not self.有立繪:
            logging.warning(f'新建了沒有立繪的角色「{名字}」')
        if 立繪表:
            self.衣圖層 = 立繪表['衣']
            self.顏圖層 = 立繪表['顏']
            self.固有縮放 = 立繪表.get('縮放', 1)
            self.現顏 = None
            self.現衣 = None
            self.現特效 = None

        assert self.名字 not in 角色表
        角色表[self.名字] = self

    @property
    def 現衣圖層(self):
        try:
            return self.衣圖層[self.現衣 or '_默認']
        except Exception:
            logging.warning(f'衣「{self.現衣}」沒有配置。')
            return []

    @property
    def 現顏圖層(self):
        try:
            return self.顏圖層[self.現顏 or '_默認']
        except Exception:
            logging.warning(f'顏「{self.現顏}」沒有配置。')
            return []

    def __repr__(self):
        return f'角色{"|"+self.顯示名字 if self.顯示名字 else ""}({self.名字}->[衣:{self.衣圖層}],[顏:{self.顏圖層}])'



import logging
import copy

import cloudpickle as pickle
import yaml
import sass
import liber
from rimo_utils import good_open

from . import 鏡頭
from . import 角色
from . import 命令
from . import 箱庭



class 狀態:
    def __init__(self):
        self.話語 = None      # str
        self.名字 = None      # str
        self.語者 = None      # str
        self.人物 = None      # str
        self.背景 = None      # [圖名, 淡入時間, 位置, 漸變方法]
        self.背景音樂 = None   # [音樂名, 音量]
        self.效果音 = None    # [音樂名, 音量]
        self.插入圖 = None     # str
        self.cg = None        # [圖名, 淡入時間, 漸變方法]
        self.視頻 = None      # [視頻名, 可以跳過]
        self.js = None        # str
        self.html = None      # str
        self.選項 = []
        self.特效表 = {}
        self.額外信息 = ()
        self.源 = []

    def 導出(self):
        鏡頭.語者 = self.語者
        if self.人物:
            立繪 = 鏡頭.查詢(self.人物).拆解()
        else:
            立繪 = []
        if self.語者:
            表情 = 角色.取角色(self.語者).現顏
        else:
            表情 = ''
        快照 = {
            '話語': self.話語,
            '名字': self.名字,
            '立繪': 立繪,
            '表情': 表情,
            '語者': self.語者,
            '背景': self.背景,
            '背景音樂': self.背景音樂,
            '效果音': self.效果音,
            '插入圖': self.插入圖,
            'cg': self.cg,
            '視頻': self.視頻,
            'js': self.js,
            'html': self.html,
            '選項': [i[0] for i in self.選項],
            '特效表': copy.deepcopy(self.特效表),
            '額外信息': self.額外信息,
            '源': self.源,
        }
        self.清除臨時狀態()
        return 快照

    def 重置(self):
        self.__init__()

    def 清除臨時狀態(self):
        self.js = None
        self.插入圖 = None
        self.效果音 = None
        self.視頻 = None
        self.額外信息 = ()
        self.源 = []


class 劇本:
    def __init__(self, 內容, 名):
        self.內容 = 內容
        self.指針 = 0
        self.名 = 名

    def 下一句(self):
        if self.指針 >= len(self.內容):
            return None
        r = self.內容[self.指針]
        self.指針 += 1
        return r


class 讀者:
    def __init__(self, 初始劇本):
        self.劇本棧 = [self.編譯(初始劇本)]
        self.狀態 = 狀態()
        self.狀態.重置()
        self.箱庭 = 箱庭.箱庭(self)
        鏡頭.鏡頭對應 = {}

    @property
    def 劇本文件(self):
        return self.劇本棧[-1]

    def 下一句(self):
        if not self.劇本棧:
            return {'旁白': '<small>【演出結束了】</small>', '類型': '終焉'}
        s = self.劇本文件.下一句()
        if s:
            return s
        else:
            self.劇本棧.pop()
            return self.下一句()

    def 編譯(self, s):
        with good_open(s) as f:
            return 劇本(liber.load(f), s)

    def 步進(self, 防止終焉=False):
        s = self.下一句()
        if s['類型'] == '終焉' and 防止終焉:
            return True
        self.狀態.源.append(copy.copy(s))
        讀者句控制(self, s['類型'], s)

    def 迭代器(self):
        while True:
            self.步進()
            s = self.狀態.導出()
            if s['額外信息'] and s['額外信息'][0] == '終焉':
                break
            else:
                yield s

    def 從一而終(self, 劇本):
        self.__init__(劇本)
        logging.debug(self.劇本文件.內容)
        while True:
            if self.步進(防止終焉=True):
                break

    def 存檔(self, 路徑, 存檔信息=None):
        虛擬機狀態 = {
            '讀者狀態': self.狀態,
            '角色表': 角色.角色表,
            '鏡頭對應': 鏡頭.鏡頭對應,
            '劇本棧': self.劇本棧,
            '箱庭': self.箱庭,
        }
        with open(路徑, 'wb') as f:
            pickle.dump({
                '虛擬機狀態': 虛擬機狀態,
                '存檔信息': 存檔信息
            }, f)

    def 讀檔(self, 路徑):
        try:
            with open(路徑, 'rb') as f:
                data = pickle.load(f)['虛擬機狀態']
                角色.角色表 = data['角色表']
                鏡頭.鏡頭對應 = data['鏡頭對應']
                self.狀態 = data['讀者狀態']
                self.狀態.額外信息 = ('load',)
                self.劇本棧 = data['劇本棧']
                self.箱庭 = data['箱庭']
        except Exception as e:
            logging.warning('讀檔失敗……')
            logging.exception(e)


class 讀者句控制:
    def __init__(self, 讀者, 類型, 參數表):
        del 參數表['類型']
        if '縮進數' in 參數表:
            del 參數表['縮進數']
        if '之後的空白' in 參數表:
            del 參數表['之後的空白']
        logging.debug(類型)
        logging.debug(參數表)
        讀者句控制.__getattribute__(self, 類型)(讀者, **參數表)

    @staticmethod
    def 註釋(讀者, 註釋):
        讀者.步進()

    @staticmethod
    def 躍點(讀者, 躍點):
        讀者.步進()

    @staticmethod
    def 函數調用(讀者, 函數, 參數表, 原文):
        命令.命令(函數, 參數表, 原文).執行(讀者)
        if not 讀者.狀態.選項:
            讀者.步進()

    @staticmethod
    def 插入代碼(讀者, 代碼類型, 代碼內容):
        if 代碼類型 in ['', 'py', 'python']:
            exec(代碼內容, 讀者.箱庭.箱庭內容)
        elif 代碼類型 in ['js', 'javascript']:
            讀者.狀態.js = 代碼內容
        elif 代碼類型 in ['html']:
            讀者.狀態.html = 代碼內容
        elif 代碼類型 in ['css']:
            讀者.狀態.html = f'<style>{代碼內容}</style>'
        elif 代碼類型 in ['sass']:
            讀者.狀態.html = f'<style>{sass.compile(string=代碼內容, indented=True)}</style>'
        elif 代碼類型 in ['scss']:
            讀者.狀態.html = f'<style>{sass.compile(string=代碼內容)}</style>'
        else:
            raise Exception(f'『{代碼類型}』代碼類型不明白。')
        if not 讀者.狀態.選項:
            讀者.步進()

    @staticmethod
    def 插入圖(讀者, 插入圖):
        logging.debug('插入圖: %s' % 插入圖)
        讀者.狀態.插入圖 = 插入圖

    @staticmethod
    def 終焉(讀者, 旁白):
        讀者.狀態.額外信息 = ('終焉',)
        讀者.狀態.話語 = 旁白
        讀者.狀態.名字 = ''
        讀者.狀態.語者 = ''

    @staticmethod
    def 鏡頭(讀者, 鏡頭符號, 內容):
        if 鏡頭符號 == '+':
            try:
                d = yaml.load(內容)
                鏡頭.生成鏡頭(d)
            except Exception as e:
                logging.exception(e)
                logging.warning(f'鏡頭「{內容}」的內容不正確。')
            讀者.步進()
        elif 鏡頭符號 == '-':
            try:
                鏡頭.解除鏡頭(yaml.load(內容))
            except Exception as e:
                logging.exception(e)
                logging.warning(f'鏡頭「{內容}」的內容不正確。')
            讀者.步進()

    @staticmethod
    def 旁白(讀者, 旁白):
        讀者.狀態.話語 = 旁白
        讀者.狀態.名字 = ''
        讀者.狀態.語者 = ''

    @staticmethod
    def 人物操作(讀者, 人物名, 目標, 操作符):
        if 操作符 == '+':
            角色.取角色(人物名).現衣 = 目標
        if 操作符 == '|':
            角色.取角色(人物名).顯示名字 = 目標
        讀者.步進()

    @staticmethod
    def 人物對話(讀者, 名, 顏, 語, 代, 特效):
        人物 = 讀者句控制._表情變化(讀者, 名, 顏, 代, 特效)
        讀者.狀態.話語 = 語
        讀者.狀態.名字 = 代 or 人物.顯示名字 or 名
        讀者.狀態.語者 = 名
        logging.debug([名, 代, 顏, 語].__str__())

    @staticmethod
    def 人物表情(讀者, 名, 顏, 代, 特效):
        人物 = 讀者句控制._表情變化(讀者, 名, 顏, 代, 特效)
        logging.debug([名, 代, 顏].__str__())
        讀者.步進()

    @staticmethod
    def 選項(讀者, 選項名, 文件, 位置):
        讀者.狀態.選項.append([選項名, lambda: 讀者.箱庭.棧跳轉(文件, 位置)])
        if 讀者.劇本文件.指針 < len(讀者.劇本文件.內容):
            if 讀者.劇本文件.內容[讀者.劇本文件.指針]['類型'] == '選項':
                讀者.步進()

    @staticmethod
    def _表情變化(讀者, 名, 顏, 代, 特效):
        人物 = 角色.取角色(名)
        人物.現顏 = 顏
        if 特效:
            人物.現特效 = [特效]
        else:
            人物.現特效 = []
        if 鏡頭.查詢(名) and 讀者.狀態.人物 != 名:
            讀者.狀態.人物 = 名
        return 人物



import os
import logging
    

def yaml(文件名, 默認=[]):
    import yaml
    return 加載(文件名, yaml.safe_load, 默認)
    
    
def 加載(文件名, 加載方法, 默認):
    if os.path.isfile(文件名):
        try:
            with open(文件名, encoding='utf8') as f:
                內容 = 加載方法(f)
            return 內容
        except Exception as e:
            logging.exception(e)
            logging.warning(f'「{文件名}」沒能讀入。')
    return 默認
    
    
    


import setuptools


setuptools.setup(
    name='librian',
    version='0.0.1',
    author='RimoChan',
    author_email='the@librian.net',
    description='librian',
    long_description=open('readme.md', encoding='utf8').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/RimoChan/librian',
    packages=[
        'librian', 
        'librian.librian_util',
        'librian.librian本體',
        'librian.librian本體.librian虛擬機',
    ],
    package_data={
        'librian.librian本體': [
            '前端/*', '前端/dist/*', '前端/默認標題畫面/*', '前端/素材/*', '前端/主題/*', 
            '資源', '資源/導出pdf用',
            '配置.yaml',
        ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'liber>=1.0.1',
        'rimo_utils>=1.4.1',
        'cefpython3>=66.0',
        'wxPython>=4.0.4',
        'cloudpickle>=1.2.2',
        'opencc>=1.1.1',
        'PyYAML>=5.2',
        'fire>=0.2.1',
        'dulwich>=0.19.14',
        'requests>=2.24.0',
        'libsass>=0.20.0',
    ],
    python_requires='>=3.6, <=3.7',
)



import copy

from . import 角色


def 人物拆解(人名: str, 镜头參數):
    人 = 角色.取角色(人名)

    if not 人.有立繪:
        return None

    位置 = copy.deepcopy(镜头參數['位置'])
    語 = 镜头參數['語']

    if 人.使用png:
        return {
            '使用png': True,
            '位置': 位置,
            '特效': 人.現特效,
            '名字': 人名,
        }

    位置[2] *= 人.固有縮放

    衣配件 = 人.現衣圖層[::-1]
    顏 = 人.現顏圖層
    if isinstance(顏, dict):
        if 語:
            顏 = 顏['_語']
        else:
            顏 = 顏['_默認']
    顏配件 = 顏[::-1]

    return {
        '圖層': 衣配件 + 顏配件,
        '位置': 位置,
        '特效': 人.現特效,
        '名字': 人名,
    }
